/*
 * arch/arm/mach-ep93xx/arch.c
 * Atmark Techno Armadillo2x0, Armadillo-9 support.
 *
 * Copyright (C) 2009 Atmark Techno, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/platform_device.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/cfi.h>
#include <linux/mtd/partitions.h>
#include <linux/i2c/eeprom.h>
#include <linux/if_ether.h>
#include <linux/leds.h>
#include <linux/gpio.h>
#include <linux/i2c-gpio.h>
#include <asm/hardware.h>
#include <asm/arch/platform.h>
#include <asm/arch/memmap.h>
#include <asm/arch/mtd.h>
#include <asm/arch/ep93xx-regs.h>
#include <asm/mach/flash.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>

#define MTD_PART(_name, _size, _offset, _mask_flags) \
	{                                            \
		.name		= _name,             \
		.size		= _size,             \
		.offset		= _offset,           \
		.mask_flags	= _mask_flags,       \
	}

#define GPIO_PIN(_name, _gpio)                         	\
	{                                              	\
		.name = _name,			       	\
		.value_name = _name EP93XX_GPIO_VALUE_SUFFIX,		\
		.direction_name = _name EP93XX_GPIO_DIRECTION_SUFFIX,	\
		.debounce_name = _name EP93XX_GPIO_DEBOUNCE_SUFFIX,	\
		.gpio = _gpio,		                \
	}

#define LED_PIN(_name, _default_trigger, _gpio, _active_low) \
	{                                                    \
		.name            = _name,                    \
		.default_trigger = _default_trigger,         \
		.gpio            = _gpio,                    \
		.active_low      = _active_low,              \
	}

#define I2C_GPIO_DATA(_sda, _scl, _delay)             \
	{                                             \
		.sda_pin = _sda,                      \
		.scl_pin = _scl,                      \
		.udelay = _delay,                     \
	}

#if defined(CONFIG_MTD_ARMADILLO2X0_NOR) || \
    defined(CONFIG_MTD_NOR_ARMADILLO2X0_NOR_MODULE)
struct mtd_partition mtd_nor_partitions_4MB[] = {
	MTD_PART("bootloader", 8*8*1024,    0,
		 MTD_WRITEABLE /* force read-only */),
	MTD_PART("kernel",     23*64*1024,  MTDPART_OFS_APPEND, 0),
	MTD_PART("userland",   39*64*1024,  MTDPART_OFS_APPEND, 0),
	MTD_PART("config",     8*8*1024,    MTDPART_OFS_APPEND, 0),
};

struct mtd_partition mtd_nor_partitions_8MB[] = {
	MTD_PART("bootloader", 8*8*1024,    0,
		 MTD_WRITEABLE /* force read-only */),
	MTD_PART("kernel",     23*64*1024,  MTDPART_OFS_APPEND, 0),
	MTD_PART("userland",   103*64*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("config",     8*8*1024,    MTDPART_OFS_APPEND, 0),
};

static int mtd_updata_partitions(struct map_info *map,
				 struct flash_platform_data *plat)
{
	struct cfi_private *cfi = map->fldrv_priv;

	switch (cfi->cfiq->DevSize) {
	case 22: /* 4MB */
		plat->parts	= mtd_nor_partitions_4MB;
		plat->nr_parts	= ARRAY_SIZE(mtd_nor_partitions_4MB);
		break;
	case 23: /* 8MB */
		plat->parts	= mtd_nor_partitions_8MB;
		plat->nr_parts	= ARRAY_SIZE(mtd_nor_partitions_8MB);
		break;
	default:
		printk("Not support flash-size.\n");
		return -1;
	}

	return 0;
}

static struct ep93xx_flash_private_data ep93xx_nor_data = {
	.plat = {
		.map_name	= "cfi_probe",
		.width		= 2,
		.parts		= mtd_nor_partitions_8MB,
		.nr_parts	= ARRAY_SIZE(mtd_nor_partitions_8MB),
	},
	.updata_partitions	= mtd_updata_partitions,
	.map_name		= "armadillo2x0-nor",
};

static struct resource ep93xx_nor_resource = {
	.start		= NOR_BASE_ADDR,
	.end		= NOR_BASE_ADDR + NOR_SIZE - 1,
	.flags		= IORESOURCE_MEM,
};

static struct platform_device ep93xx_nor_device = {
	.name		= "armadillo2x0-nor",
	.id		= 0,
	.dev		= {
		.platform_data	= &ep93xx_nor_data,
	},
	.num_resources	= 1,
	.resource	= &ep93xx_nor_resource,
};

static inline void ep93xx_add_nor(void)
{
	platform_device_register(&ep93xx_nor_device);
}
#else
static inline void ep93xx_add_nor(void) {}
#endif /* defined(CONFIG_MTD_ARMADILLO2X0_NOR) || 
	  defined(CONFIG_MTD_NOR_ARMADILLO2X0_NOR_MODULE) */

#if defined(CONFIG_MTD_NAND_EP93XX) || defined(CONFIG_MTD_NAND_EP93XX_MODULE)
static struct mtd_partition mtd_nand_partitions[] = {
	MTD_PART("nand", MTDPART_SIZ_FULL, 0, 0),
};

static struct ep93xx_nand_platdata ep93xx_nand_platdata = {
	.name         = "armadillo2x0-nand",
	.gpio_nce     = EP93XX_GPIO_LINE_D(4),
	.gpio_cle     = EP93XX_GPIO_LINE_D(5),
	.gpio_ale     = EP93XX_GPIO_LINE_D(6),
	.gpio_rdy     = ARCH_NR_GPIOS, /* don't use gpio_rdy, so set invalid gpio. */
	.parts	      = mtd_nand_partitions,
	.num_parts    = ARRAY_SIZE(mtd_nand_partitions),
	.chip_delay   = 25,
};

static struct resource ep93xx_nand_resource = {
	.start		= NAND_BASE_ADDR,
	.end		= NAND_BASE_ADDR + NAND_SIZE - 1,
	.flags		= IORESOURCE_MEM,
};

static struct platform_device ep93xx_nand_device = {
	.name		= "ep93xx-nand",
	.id		= -1,
	.dev		= {
		.platform_data = &ep93xx_nand_platdata,
	},
	.num_resources	= 1,
	.resource	= &ep93xx_nand_resource,
};

static inline void ep93xx_add_nand(void)
{
	platform_device_register(&ep93xx_nand_device);
}
#else
static inline void ep93xx_add_nand(void) {}
#endif /* defined(CONFIG_MTD_NAND_EP93XX) || defined(CONFIG_MTD_NAND_EP93XX_MODULE) */

#if defined(CONFIG_I2C_GPIO) || defined(CONFIG_I2C_GPIO_MODULE)
static struct i2c_gpio_platform_data i2c_platdata[] = {
	I2C_GPIO_DATA(EP93XX_GPIO_LINE_B(4), EP93XX_GPIO_LINE_B(5), 2),
#if defined(CONFIG_RTC_DRV_S353XXA)
#if defined(CONFIG_MACH_ARMADILLO220)
	I2C_GPIO_DATA(EP93XX_GPIO_LINE_D(7), EP93XX_GPIO_LINE_F(5), 2),
#elif defined(CONFIG_MACH_ARMADILLO9)
	I2C_GPIO_DATA(EP93XX_GPIO_LINE_B(4), EP93XX_GPIO_LINE_B(5), 2),
#endif
#endif /* defined(CONFIG_RTC_DRV_S353XXA) */
};

static struct platform_device ep93xx_i2c_device[] = {
	{
		.name               = "i2c-gpio",
		.id                 = 0,
		.dev		= {
			.platform_data = &i2c_platdata[0],
		}
	},
#if defined(CONFIG_RTC_DRV_S353XXA)
	{
		.name               = "i2c-gpio",
		.id                 = 1,
		.dev		= {
			.platform_data = &i2c_platdata[1],
		}
	},
#endif /* defined(CONFIG_RTC_DRV_S353XXA) */
};

static inline void ep93xx_add_i2c(void)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(ep93xx_i2c_device); i++)
		platform_device_register(&ep93xx_i2c_device[i]);
}
#else
static inline void ep93xx_add_i2c(void) {}
#endif /* defined(CONFIG_I2C_GPIO) || defined(CONFIG_I2C_GPIO_MODULE) */

#if defined(CONFIG_EP93XX_ETH) || defined(CONFIG_EP93XX_ETH_MODULE)
static int set_mac(struct ep93xx_eth_data *data)
{
	char mac[ETH_ALEN];

	if (eeprom_read_by_address(0x50, mac, 0, ETH_ALEN) == ETH_ALEN) {
		memcpy(data->dev_addr, mac, ETH_ALEN);
		return 0;
	}

	return -ENODEV;
}

static struct ep93xx_eth_data ep93xx_eth_data = {
	.phy_id			= 0,
	.set_mac		= set_mac,
};

static struct resource ep93xx_eth_resource[] = {
	{
		.start	= EP93XX_ETHERNET_PHYS_BASE,
		.end	= EP93XX_ETHERNET_PHYS_BASE + 0xffff,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= IRQ_EP93XX_ETHERNET,
		.end	= IRQ_EP93XX_ETHERNET,
		.flags	= IORESOURCE_IRQ,
	}
};

static struct platform_device ep93xx_eth_device = {
	.name		= "ep93xx-eth",
	.id		= -1,
	.dev		= {
		.platform_data	= &ep93xx_eth_data,
	},
	.num_resources	= ARRAY_SIZE(ep93xx_eth_resource),
	.resource	= ep93xx_eth_resource,
};

static inline void ep93xx_add_eth(void)
{
	platform_device_register(&ep93xx_eth_device);
}
#else
static inline void ep93xx_add_eth() {}
#endif /* defined(CONFIG_EP93XX_ETH) || defined(CONFIG_EP93XX_ETH_MODULE) */

#if defined(CONFIG_GPIO_EP93XX_SYSFS) || defined(CONFIG_GPIO_EP93XX_SYSFS_MODULE)
static struct ep93xx_gpio_pin gpio_pins[] = {
	GPIO_PIN("gpio0",  EP93XX_GPIO_LINE_A(4)),
	GPIO_PIN("gpio1",  EP93XX_GPIO_LINE_A(5)),
	GPIO_PIN("gpio2",  EP93XX_GPIO_LINE_A(6)),
	GPIO_PIN("gpio3",  EP93XX_GPIO_LINE_A(7)),
	GPIO_PIN("gpio4",  EP93XX_GPIO_LINE_B(0)),
	GPIO_PIN("gpio5",  EP93XX_GPIO_LINE_B(1)),
	GPIO_PIN("gpio6",  EP93XX_GPIO_LINE_B(2)),
	GPIO_PIN("gpio7",  EP93XX_GPIO_LINE_B(3)),
#if defined(CONFIG_MACH_ARMADILLO220)
	GPIO_PIN("gpio8",  EP93XX_GPIO_LINE_D(0)),
	GPIO_PIN("gpio9",  EP93XX_GPIO_LINE_D(1)),
	GPIO_PIN("gpio10", EP93XX_GPIO_LINE_D(2)),
	GPIO_PIN("gpio11", EP93XX_GPIO_LINE_D(3)),
	GPIO_PIN("gpio12", EP93XX_GPIO_LINE_C(0)),
	GPIO_PIN("gpio13", EP93XX_GPIO_LINE_C(1)),
	GPIO_PIN("gpio14", EP93XX_GPIO_LINE_C(2)),
	GPIO_PIN("gpio15", EP93XX_GPIO_LINE_C(3)),
#elif defined(CONFIG_MACH_ARMADILLO9)
	GPIO_PIN("gpio8",  EP93XX_GPIO_LINE_D(4)),
	GPIO_PIN("gpio9",  EP93XX_GPIO_LINE_D(5)),
	GPIO_PIN("gpio10", EP93XX_GPIO_LINE_D(6)),
	GPIO_PIN("gpio11", EP93XX_GPIO_LINE_D(7)),
#endif
};

static struct ep93xx_gpio_platform_data ep93xx_gpio_plat_data = {
	.gpios     = gpio_pins,
	.num_gpios = ARRAY_SIZE(gpio_pins),
};

static struct platform_device ep93xx_gpio_device = {
	.name = "ep93xx-gpio",
	.id = 0,
	.dev = {
		.platform_data = &ep93xx_gpio_plat_data,
	},
};

static inline void ep93xx_add_gpio(void)
{
	platform_device_register(&ep93xx_gpio_device);
}
#else
static inline void ep93xx_add_gpio(void) {}
#endif /* defined(CONFIG_GPIO_EP93XX_SYSFS) || defined(CONFIG_GPIO_EP93XX_SYSFS_MODULE) */

#if defined(CONFIG_LEDS_GPIO) || defined(CONFIG_LEDS_GPIO_MODULE)
static struct gpio_led led_pins[] = {
#if defined(CONFIG_MACH_ARMADILLO210)
	LED_PIN("red", "default-on", EP93XX_GPIO_LINE_E(0), 0),
	LED_PIN("green", "default-on", EP93XX_GPIO_LINE_E(1), 0),
#else
	LED_PIN("red", "default-on", EP93XX_GPIO_LINE_C(7), 0),
	LED_PIN("green", "default-on", EP93XX_GPIO_LINE_E(0), 0),
#endif /* defined(CONFIG_MACH_ARMADILLO210) */
};

static struct gpio_led_platform_data ep93xx_led_platform_data = {
	.leds     = led_pins,
	.num_leds = ARRAY_SIZE(led_pins),
};

static struct platform_device ep93xx_led_device = {
	.name	= "armadillo2x0-led",
	.id	= 0,
	.dev = {
		.platform_data = &ep93xx_led_platform_data,
	},
};

static inline void ep93xx_add_led(void)
{
	platform_device_register(&ep93xx_led_device);
}
#else
static inline void ep93xx_add_led(void) {}
#endif /* defined(CONFIG_LEDS_GPIO) || defined(CONFIG_LEDS_GPIO_MODULE) */

#if defined(CONFIG_INPUT_EP93XX_SW) || defined(CONFIG_INPUT_EP93XX_SW_MODULE)
static struct ep93xx_gpio_pin sw_pins[] = {
	GPIO_PIN("tactsw1",  EP93XX_GPIO_LINE_F(7)),
};

static struct ep93xx_gpio_platform_data ep93xx_sw_platform_data = {
	.gpios     = sw_pins,
	.num_gpios = ARRAY_SIZE(sw_pins),
};

static struct platform_device ep93xx_sw_device = {
	.name = "ep93xx-sw",
	.id = 0,
	.dev = {
		.platform_data = &ep93xx_sw_platform_data,
	},
};

static inline void ep93xx_add_sw(void)
{
	platform_device_register(&ep93xx_sw_device);
}
#else
static inline void ep93xx_add_sw(void) {}
#endif /* defined(CONFIG_INPUT_EP93XX_SW) || defined(CONFIG_INPUT_EP93XX_SW_MODULE) */

static void __init ep93xx_init_machine(void)
{
	ep93xx_init_devices();

	ep93xx_add_nor();
	ep93xx_add_nand();
	ep93xx_add_i2c();
	ep93xx_add_eth();
	ep93xx_add_gpio();
	ep93xx_add_led();
	ep93xx_add_sw();
}

#if defined(CONFIG_MACH_ARMADILLO220)
MACHINE_START(ARMADILLO220, "Atmark Techno Armadillo220")
	.phys_io	= EP93XX_APB_PHYS_BASE,
	.io_pg_offst	= ((EP93XX_APB_VIRT_BASE) >> 18) & 0xfffc,
	.boot_params	= 0xc0000100,
	.map_io		= ep93xx_map_io,
	.init_irq	= ep93xx_init_irq,
	.timer		= &ep93xx_timer,
	.init_machine	= ep93xx_init_machine,
MACHINE_END
#endif

#if defined(CONFIG_MACH_ARMADILLO9)
MACHINE_START(ARMADILLO9, "Atmark Techno Armadillo9")
	.phys_io	= EP93XX_APB_PHYS_BASE,
	.io_pg_offst	= ((EP93XX_APB_VIRT_BASE) >> 18) & 0xfffc,
	.boot_params	= 0xc0000100,
	.map_io		= ep93xx_map_io,
	.init_irq	= ep93xx_init_irq,
	.timer		= &ep93xx_timer,
	.init_machine	= ep93xx_init_machine,
MACHINE_END
#endif
